@@ -474,9 +474,12 @@ public:
                 results.append(r);
             }
         } else if (!query.categories().isEmpty()) {
-            QSet<QPlaceCategory> categories = query.categories().toSet();
-            foreach (const QPlace &place, m_places) {
-                if (place.categories().toSet().intersect(categories).isEmpty())
+            const auto &categoryList = query.categories();
+            const QSet<QPlaceCategory> categories(categoryList.cbegin(), categoryList.cend());
+            for (const QPlace &place : qAsConst(m_places)) {
+                const auto &placeCategoryList = place.categories();
+                const QSet<QPlaceCategory> placeCategories(placeCategoryList.cbegin(), placeCategoryList.cend());
+                if (!placeCategories.intersects(categories))
                     continue;
 
                 QPlaceResult r;
@@ -571,11 +574,8 @@ public:
             m_categories.insert(category.categoryId(), category);
             QStringList children = m_childCategories.value(parentId);
 
-            QMutableHashIterator<QString, QStringList> i(m_childCategories);
-            while (i.hasNext()) {
-                i.next();
-                i.value().removeAll(category.categoryId());
-            }
+            for (QStringList &c : m_childCategories)
+                c.removeAll(category.categoryId());
 
             if (!children.contains(category.categoryId())) {
                 children.append(category.categoryId());
@@ -611,11 +611,8 @@ public:
         } else {
             m_categories.remove(categoryId);
 
-            QMutableHashIterator<QString, QStringList> i(m_childCategories);
-            while (i.hasNext()) {
-                i.next();
-                i.value().removeAll(categoryId);
-            }
+            for (auto &c : m_childCategories)
+                c.removeAll(categoryId);
         }
 
         QMetaObject::invokeMethod(reply, "emitFinished", Qt::QueuedConnection);
@@ -634,9 +631,7 @@ public:
 
     QString parentCategoryId(const QString &categoryId) const override
     {
-        QHashIterator<QString, QStringList> i(m_childCategories);
-        while (i.hasNext()) {
-            i.next();
+        for (auto i = m_childCategories.cbegin(), end = m_childCategories.cend(); i != end; ++i) {
             if (i.value().contains(categoryId))
                 return i.key();
         }
